@@ -18,9 +18,9 @@
 #ifndef JUK_CACHE_H
 #define JUK_CACHE_H
 
-#include <QtCore/QDataStream>
-#include <QtCore/QFile>
-#include <QtCore/QBuffer>
+#include <QDataStream>
+#include <QFile>
+#include <QBuffer>
 
 class Playlist;
 class PlaylistCollection;
@@ -58,8 +58,12 @@ public:
     static void loadPlaylists(PlaylistCollection *collection);
     static void savePlaylists(const PlaylistList &playlists);
 
+    static void ensureAppDataStorageExists();
     static bool cacheFileExists();
 
+    static QString fileHandleCacheFileName();
+    static QString playlistsCacheFileName();
+
     bool prepareToLoadCachedItems();
     FileHandle loadNextCachedItem();
 
