@@ -26,12 +26,12 @@
 
 #include <Phonon/AudioOutput>
 #include <Phonon/MediaObject>
+#include <Phonon/MediaSource>
 
 #include <QPixmap>
 #include <QTimer>
 #include <QUrl>
 
-#include <math.h>
 #include <algorithm>
 
 #include "playlistinterface.h"
@@ -50,6 +50,18 @@ using namespace ActionCollection;
 
 enum PlayerManagerStatus { StatusStopped = -1, StatusPaused = 1, StatusPlaying = 2 };
 
+////////////////////////////////////////////////////////////////////////////////
+// static functions
+////////////////////////////////////////////////////////////////////////////////
+static void updateWindowTitle(const FileHandle &file)
+{
+    JuK::JuKInstance()->setWindowTitle(i18nc(
+        "%1 is the artist and %2 is the title of the currently playing track.", 
+        "%1 - %2 :: JuK",
+        file.tag()->artist(),
+        file.tag()->title()));
+}
+
 ////////////////////////////////////////////////////////////////////////////////
 // protected members
 ////////////////////////////////////////////////////////////////////////////////
@@ -57,14 +69,13 @@ enum PlayerManagerStatus { StatusStopped = -1, StatusPaused = 1, StatusPlaying =
 PlayerManager::PlayerManager() :
     QObject(),
     m_playlistInterface(nullptr),
-    m_setup(false)
+    m_output(new Phonon::AudioOutput(Phonon::MusicCategory, this)),
+    m_media( new Phonon::MediaObject(this)),
+    m_audioPath(Phonon::createPath(m_media, m_output))
 {
-// This class is the first thing constructed during program startup, and
-// therefore has no access to the widgets needed by the setup() method.
-// Since the setup() method will be called indirectly by the player() method
-// later, just disable it here. -- mpyne
-//    setup();
+    setupAudio();
     new PlayerAdaptor(this);
+    QDBusConnection::sessionBus().registerObject("/Player", this);
 }
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -73,42 +84,27 @@ PlayerManager::PlayerManager() :
 
 bool PlayerManager::playing() const
 {
-    if(!m_setup)
-        return false;
-
     Phonon::State state = m_media->state();
     return (state == Phonon::PlayingState || state == Phonon::BufferingState);
 }
 
 bool PlayerManager::paused() const
 {
-    if(!m_setup)
-        return false;
-
     return m_media->state() == Phonon::PausedState;
 }
 
 bool PlayerManager::muted() const
 {
-    if(!m_setup)
-        return false;
-
     return m_output->isMuted();
 }
 
 float PlayerManager::volume() const
 {
-    if(!m_setup)
-        return 1.0;
-
     return m_output->volume();
 }
 
 int PlayerManager::status() const
 {
-    if(!m_setup)
-        return StatusStopped;
-
     if(paused())
         return StatusPaused;
 
@@ -130,25 +126,16 @@ int PlayerManager::currentTime() const
 
 int PlayerManager::totalTimeMSecs() const
 {
-    if(!m_setup)
-        return 0;
-
     return m_media->totalTime();
 }
 
 int PlayerManager::currentTimeMSecs() const
 {
-    if(!m_setup)
-        return 0;
-
     return m_media->currentTime();
 }
 
 bool PlayerManager::seekable() const
 {
-    if(!m_setup)
-        return false;
-
     return m_media->isSeekable();
 }
 
@@ -202,41 +189,16 @@ void PlayerManager::setPlaylistInterface(PlaylistInterface *interface)
 
 void PlayerManager::play(const FileHandle &file)
 {
-    if(!m_setup)
-        setup();
-
-    if(!m_media || !m_playlistInterface)
+    if(!m_media || !m_playlistInterface || file.isNull())
         return;
 
-    if(file.isNull()) {
-        if(paused())
-            m_media->play();
-        else if(playing()) {
-            m_media->seek(0);
-            emit seeked(0);
-        }
-        else {
-            m_playlistInterface->playNext();
-            m_file = m_playlistInterface->currentFile();
+    m_media->setCurrentSource(QUrl::fromLocalFile(file.absFilePath()));
+    m_media->play();
 
-            if(!m_file.isNull())
-            {
-                m_media->setCurrentSource(QUrl::fromLocalFile(m_file.absFilePath()));
-                m_media->play();
+    if(m_file != file)
+        emit signalItemChanged(file);
 
-                emit signalItemChanged(m_file);
-            }
-        }
-    }
-    else {
-        m_media->setCurrentSource(QUrl::fromLocalFile(file.absFilePath()));
-        m_media->play();
-
-        if(m_file != file)
-            emit signalItemChanged(file);
-
-        m_file = file;
-    }
+    m_file = file;
 
     // Our state changed handler will perform the follow up actions necessary
     // once we actually start playing.
@@ -253,12 +215,23 @@ void PlayerManager::play(const QString &file)
 
 void PlayerManager::play()
 {
-    play(FileHandle());
+    if(paused())
+        m_media->play();
+    else if(playing()) {
+        m_media->seek(0);
+        emit seeked(0);
+    }
+    else {
+        m_playlistInterface->playNext();
+        const auto file = m_playlistInterface->currentFile();
+
+        play(file);
+    }
 }
 
 void PlayerManager::pause()
 {
-    if(!m_setup || paused())
+    if(paused())
         return;
 
     action("pause")->setEnabled(false);
@@ -268,7 +241,7 @@ void PlayerManager::pause()
 
 void PlayerManager::stop()
 {
-    if(!m_setup || !m_playlistInterface)
+    if(!m_playlistInterface)
         return;
 
     action("pause")->setEnabled(false);
@@ -287,15 +260,12 @@ void PlayerManager::stop()
 
 void PlayerManager::setVolume(float volume)
 {
-    if(!m_setup)
-        setup();
-
     m_output->setVolume(volume);
 }
 
 void PlayerManager::seek(int seekTime)
 {
-    if(!m_setup || m_media->currentTime() == seekTime)
+    if(m_media->currentTime() == seekTime)
         return;
 
     m_media->seek(seekTime);
@@ -351,35 +321,23 @@ void PlayerManager::back()
 
 void PlayerManager::volumeUp()
 {
-    if(!m_setup)
-        return;
-
     const auto newVolume = std::min(m_output->volume() + 0.04, 1.0);
     m_output->setVolume(newVolume); // 4% up
 }
 
 void PlayerManager::volumeDown()
 {
-    if(!m_setup)
-        return;
-
     const auto newVolume = std::max(m_output->volume() - 0.04, 0.0);
     m_output->setVolume(newVolume); // 4% down
 }
 
 void PlayerManager::setMuted(bool m)
 {
-    if(!m_setup)
-        return;
-
     m_output->setMuted(m);
 }
 
 bool PlayerManager::mute()
 {
-    if(!m_setup)
-        return false;
-
     bool newState = !muted();
     setMuted(newState);
     return newState;
@@ -459,12 +417,7 @@ void PlayerManager::slotStateChanged(Phonon::State newstate, Phonon::State)
             action("forwardAlbum")->setEnabled(true);
         action("back")->setEnabled(true);
 
-        JuK::JuKInstance()->setWindowTitle(i18nc(
-            "%1 is the artist and %2 is the title of the currently playing track.", 
-            "%1 - %2 :: JuK",
-            m_file.tag()->artist(),
-            m_file.tag()->title()));
-
+        updateWindowTitle(m_file);
         emit signalPlay();
     }
 }
@@ -493,50 +446,21 @@ void PlayerManager::slotVolumeChanged(qreal volume)
 // private members
 ////////////////////////////////////////////////////////////////////////////////
 
-void PlayerManager::setup()
+void PlayerManager::setupAudio()
 {
-    if(m_setup)
-        return;
-
-    m_setup = true;
-
-    // All of the actions required by this class should be listed here.
-    if(!action("pause") ||
-       !action("stop") ||
-       !action("back") ||
-       !action("forwardAlbum") ||
-       !action("forward") ||
-       !action("trackPositionAction"))
-    {
-        qCWarning(JUK_LOG) << "Could not find all of the required actions.";
-        return;
-    }
-
     using namespace Phonon;
-    m_output = new AudioOutput(MusicCategory, this);
     connect(m_output, &AudioOutput::mutedChanged,  this, &PlayerManager::slotMutedChanged);
     connect(m_output, &AudioOutput::volumeChanged, this, &PlayerManager::slotVolumeChanged);
 
-    m_media = new MediaObject(this);
-    m_audioPath = createPath(m_media, m_output);
-
     connect(m_media, &MediaObject::stateChanged, this, &PlayerManager::slotStateChanged);
+    connect(m_media, &MediaObject::currentSourceChanged, this, &PlayerManager::trackHasChanged);
     connect(m_media, &MediaObject::totalTimeChanged, this, &PlayerManager::slotLength);
     connect(m_media, &MediaObject::tick, this, &PlayerManager::slotTick);
+    connect(m_media, &MediaObject::aboutToFinish, this, &PlayerManager::trackAboutToFinish);
     connect(m_media, &MediaObject::finished, this, &PlayerManager::slotFinished);
     connect(m_media, &MediaObject::seekableChanged, this, &PlayerManager::slotSeekableChanged);
 
     m_media->setTickInterval(100);
-
-    // initialize action states
-
-    action("pause")->setEnabled(false);
-    action("stop")->setEnabled(false);
-    action("back")->setEnabled(false);
-    action("forward")->setEnabled(false);
-    action("forwardAlbum")->setEnabled(false);
-
-    QDBusConnection::sessionBus().registerObject("/Player", this);
 }
 
 QString PlayerManager::randomPlayMode() const
@@ -558,4 +482,36 @@ void PlayerManager::setRandomPlayMode(const QString &randomMode)
         action<KToggleAction>("disableRandomPlay")->setChecked(true);
 }
 
+void PlayerManager::trackHasChanged(const Phonon::MediaSource &newSource)
+{
+    if(newSource.type() == Phonon::MediaSource::Url) {
+        const auto item = CollectionList::instance()->lookup(newSource.url().path());
+        if(item) {
+            const auto newFile = item->file();
+            if(m_file != newFile)
+                emit signalItemChanged(newFile);
+            m_file = newFile;
+            updateWindowTitle(m_file);
+            emit seeked(0);
+        }
+    } else {
+        qCWarning(JUK_LOG) << "Track has changed so something we didn't set???";
+        return;
+    }
+}
+
+void PlayerManager::trackAboutToFinish()
+{
+    // Called when playback is in progress and a track is about to finish, gives us a
+    // chance to keep audio playback going without Phonon entering StoppedState
+    if(!m_playlistInterface)
+        return;
+
+    m_playlistInterface->playNext();
+    const auto file = m_playlistInterface->currentFile();
+
+    if(!file.isNull())
+        m_media->enqueue(QUrl::fromLocalFile(file.absFilePath()));
+}
+
 // vim: set et sw=4 tw=0 sta:
