@@ -16,7 +16,6 @@
 
 #include "playlistbox.h"
 
-#include <kiconloader.h>
 #include <kmessagebox.h>
 #include <kactioncollection.h>
 #include <ktoggleaction.h>
@@ -730,7 +729,10 @@ void PlaylistBox::Item::playingItemHasChanged()
 
 void PlaylistBox::Item::playlistItemDataHasChanged()
 {
-    listView()->slotPlaylistDataChanged();
+    // This avoids spuriously re-saving all playlists just because play queue
+    // changes.
+    if(m_playlist != listView()->upcomingPlaylist())
+        listView()->slotPlaylistDataChanged();
 }
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -743,8 +745,7 @@ void PlaylistBox::Item::init()
 
     list->setupItem(this);
 
-    int iconSize = list->viewModeIndex() == 0 ? 32 : 16;
-    setIcon(0, SmallIcon(m_iconName, iconSize));
+    setIcon(0, QIcon::fromTheme(m_iconName));
     list->addNameToDict(m_text);
 
     if(m_playlist) {
